require "net/http"
require "uri"
require "json"

module MkRepo
  module Factory
    def self.build(remote, &blk)
      Client.new(remote, &blk)
    end
  end

  class Client
    ATTRIBUTES = %i(token repo_name user)
    BASE_URLS = {
      github:    "https://api.github.com/user/repos",
      gitlab:    "https://gitlab.com/api/v3/projects",
      bitbucket: "https://api.bitbucket.org/2.0/%s/%s"
    }

    attr_reader :uri, :provider

    def initialize(provider, &blk)
      @provider = provider
      @uri = URI.parse(url % [user, repo_name])
      instance_eval(&blk)
    end

    def create!
      Net::HTTP.start(uri.host, uri.port, use_ssl: true) do |http|
        Net::HTTP::Post.new(uri.request_uri, headers).tap do |req|
          req.body = body
          req.basic_auth(user, token) if github?

          response = http.request(req)

          response_json = JSON.parse(response.body, symbolize_names: true)

          if response.kind_of?(Net::HTTPSuccess)
            $stdout.puts response_json[:clone_url]
          else
            error_message = response_json[:errors].first[:message]
            raise StandardError,
              "#{response.code} Unable to create repository: #{error_message}"
          end
        end
      end
    end

    ATTRIBUTES.each do |attr|
      define_method(attr) do |val = ""|
        if val && val != ""
          instance_variable_set("@#{attr}", val)
        end
        instance_variable_get("@#{attr}")
      end
    end

    private

    BASE_URLS.keys.each do |k|
      define_method("#{k}?") { provider == k }
    end

    def url
      BASE_URLS[provider]
    end

    def body
      { "name" => repo_name }.to_json if github?
    end

    def headers
      { "Content-Type" => "text/json" }
    end
  end
end
